﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SQLite;
using System.Windows.Forms;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region CREATE_SOURCE_TABLE_SQL

        /// <summary>
        /// CREATE_SOURCE_TABLE_SQL
        /// </summary>
        private const string CREATE_SOURCE_TABLE_SQL = @"
CREATE TABLE SOURCE_TABLE
(
    MANAGER      TEXT    NOT NULL
   ,SALES_MONTH  TEXT    NOT NULL
   ,SALES_AMOUNT INTEGER NOT NULL
   ,PRIMARY KEY(MANAGER, SALES_MONTH)
);
";

        #endregion
        #region INSERT_SOURCE_TABLE_SQL

        /// <summary>
        /// INSERT_SOURCE_TABLE_SQL
        /// </summary>
        private const string INSERT_SOURCE_TABLE_SQL = @"
INSERT INTO SOURCE_TABLE
(
    MANAGER
   ,SALES_MONTH
   ,SALES_AMOUNT
)
VALUES
(
    @MANAGER
   ,@SALES_MONTH
   ,@SALES_AMOUNT
);
";

        #endregion
        #region SELECT_COLUMN_COLUMN_VALUE_SQL

        /// <summary>
        /// SELECT_COLUMN_COLUMN_VALUE_SQL
        /// </summary>
        private const string SELECT_COLUMN_COLUMN_VALUE_SQL = "SELECT DISTINCT SALES_MONTH FROM SOURCE_TABLE ORDER BY SALES_MONTH ASC";

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            this.Load += Form_Load;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 폼 로드시 처리하기 - Form_Load(sender, e)

        /// <summary>
        /// 폼 로드시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Form_Load(object sender, EventArgs e)
        {
            using(SQLiteConnection connection = SQLiteHelper.GetConnection("Data Source=:memory:"))
            {
                connection.Open();

                // 소스 테이블을 생성한다.
                SQLiteHelper.Execute(connection, CREATE_SOURCE_TABLE_SQL);

                // 소스 테이블에 데이터를 추가한다.
                InsertSourceTableData(connection);
                
                // 행 컬럼명 리스트를 설정한다.
                List<string> rowColumnNameList = SQLiteHelper.GetStringList("MANAGER");

                // 컬럼 컬럼 값 리스트를 설정한다.
                DataTable columnColumnValueTable = SQLiteHelper.GetDataTable(connection, SELECT_COLUMN_COLUMN_VALUE_SQL);

                List<string> columnColumnValueList = SQLiteHelper.GetStringList(columnColumnValueTable, "SALES_MONTH");

                // 정렬 컬럼명 리스트를 설정한다.
                List<string> orderByColumnNameList = SQLiteHelper.GetStringList("MANAGER");

                // 피벗 합계 테이블을 생성한다.
                string pivotSummarySQL = SQLiteHelper.GetPivotSummarySQL("SOURCE_TABLE", rowColumnNameList, "SALES_MONTH", columnColumnValueList, "SALES_AMOUNT", orderByColumnNameList);

                SQLiteHelper.Execute(connection, "CREATE TABLE PIVOT_TABLE AS " + pivotSummarySQL);

                // 소스 테이블을 출력한다.
                DataTable sourceTable = SQLiteHelper.GetDataTable(connection, "SELECT * FROM SOURCE_TABLE");

                // 소스 데이터 그리드 뷰에 소스 테이블을 바인딩 한다.
                this.sourceDataGridView.DataSource = sourceTable;

                // 피벗 테이블을 출력한다.
                DataTable pivotTable = SQLiteHelper.GetDataTable(connection, "SELECT * FROM PIVOT_TABLE");

                // 피벗 데이터 그리드 뷰에 피벗 테이블을 바인딩 한다.
                this.pivotDataGridView.DataSource = pivotTable;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 소스 데이터 추가하기 - InsertSourceData(connection, lotWafer, tkinTime, stepSeq, subItemID, value)

        /// <summary>
        /// 소스 데이터 추가하기
        /// </summary>
        /// <param name="connection">연결</param>
        /// <param name="lotWafer">LOT_WF</param>
        /// <param name="tkinTime">TKIN_TIME</param>
        /// <param name="stepSeq">STEP_SEQ</param>
        /// <param name="subItemID">SUB_ITEM_ID</param>
        /// <param name="value">VALUE</param>
        /// <returns>처리 행 수</returns>
        private int InsertSourceData(SQLiteConnection connection, string manager, string salesMonth, double salesAmount)
        {
            List<SQLiteParameter> parameterList = new List<SQLiteParameter>()
            {
                new SQLiteParameter { ParameterName = "@MANAGER"     , DbType = DbType.String, Value = manager    },
                new SQLiteParameter { ParameterName = "@SALES_MONTH" , DbType = DbType.String, Value = salesMonth },
                new SQLiteParameter { ParameterName = "@SALES_AMOUNT", DbType = DbType.Double, Value = salesAmount}
            };

            int result = SQLiteHelper.Execute(connection, INSERT_SOURCE_TABLE_SQL, parameterList);

            return result;
        }

        #endregion
        #region 소스 테이블 데이터 추가하기 - InsertSourceTableData(connection)

        /// <summary>
        /// 소스 테이블 데이터 추가하기
        /// </summary>
        /// <param name="connection">연결</param>
        private void InsertSourceTableData(SQLiteConnection connection)
        {
            DateTime currentTime = DateTime.Now;

            InsertSourceData(connection, "홍길동", "01", 1000000);
            InsertSourceData(connection, "홍길동", "02", 1500000);
            InsertSourceData(connection, "홍길동", "03", 1000000);
            InsertSourceData(connection, "홍길동", "04", 2000000);
            InsertSourceData(connection, "홍길동", "05", 3000000);
            InsertSourceData(connection, "홍길동", "06", 1500000);
            InsertSourceData(connection, "홍길동", "07", 1000000);
            InsertSourceData(connection, "홍길동", "08", 1000000);
            InsertSourceData(connection, "홍길동", "09", 3000000);
            InsertSourceData(connection, "홍길동", "10", 4000000);
            InsertSourceData(connection, "홍길동", "11", 1000000);
            InsertSourceData(connection, "홍길동", "12", 1500000);

            InsertSourceData(connection, "김철수", "01",  500000);
            InsertSourceData(connection, "김철수", "02", 1500000);
            InsertSourceData(connection, "김철수", "03", 2000000);
            InsertSourceData(connection, "김철수", "04", 2500000);
            InsertSourceData(connection, "김철수", "05", 1000000);
            InsertSourceData(connection, "김철수", "06", 2500000);
            InsertSourceData(connection, "김철수", "07", 1500000);
            InsertSourceData(connection, "김철수", "08",  500000);
            InsertSourceData(connection, "김철수", "09",  500000);
            InsertSourceData(connection, "김철수", "10", 2000000);
            InsertSourceData(connection, "김철수", "11", 1000000);
            InsertSourceData(connection, "김철수", "12", 2500000);

            InsertSourceData(connection, "이영희", "01", 2500000);
            InsertSourceData(connection, "이영희", "02", 3500000);
            InsertSourceData(connection, "이영희", "03", 1000000);
            InsertSourceData(connection, "이영희", "04", 1500000);
            InsertSourceData(connection, "이영희", "05", 5000000);
            InsertSourceData(connection, "이영희", "06", 1500000);
            InsertSourceData(connection, "이영희", "07",  500000);
            InsertSourceData(connection, "이영희", "08",  500000);
            InsertSourceData(connection, "이영희", "09",  500000);
            InsertSourceData(connection, "이영희", "10",  500000);
            InsertSourceData(connection, "이영희", "11", 1000000);
            InsertSourceData(connection, "이영희", "12", 2000000);
        }

        #endregion
    }
}